# -*- coding: utf-8 -*-

# -------------------------------------------------------------------------
# This scaffolding model makes your app work on Google App Engine too
# File is released under public domain and you can use without limitations
# -------------------------------------------------------------------------

if request.global_settings.web2py_version < "2.14.1":
    raise HTTP(500, "Requires web2py 2.13.3 or newer")

# -------------------------------------------------------------------------
# if SSL/HTTPS is properly configured and you want all HTTP requests to
# be redirected to HTTPS, uncomment the line below:
# -------------------------------------------------------------------------
# request.requires_https()

# -------------------------------------------------------------------------
# app configuration made easy. Look inside private/appconfig.ini
# -------------------------------------------------------------------------
from gluon.contrib.appconfig import AppConfig

# -------------------------------------------------------------------------
# once in production, remove reload=True to gain full speed
# -------------------------------------------------------------------------
myconf = AppConfig(reload=True)

if not request.env.web2py_runtime_gae:
    # ---------------------------------------------------------------------
    # if NOT running on Google App Engine use SQLite or other DB
    # ---------------------------------------------------------------------
    db = DAL(myconf.get('db.uri'),
             pool_size=myconf.get('db.pool_size'),
             migrate_enabled=myconf.get('db.migrate'),
             check_reserved=['all'])
else:
    # ---------------------------------------------------------------------
    # connect to Google BigTable (optional 'google:datastore://namespace')
    # ---------------------------------------------------------------------
    db = DAL('google:datastore+ndb')
    # ---------------------------------------------------------------------
    # store sessions and tickets there
    # ---------------------------------------------------------------------
    session.connect(request, response, db=db)
    # ---------------------------------------------------------------------
    # or store session in Memcache, Redis, etc.
    # from gluon.contrib.memdb import MEMDB
    # from google.appengine.api.memcache import Client
    # session.connect(request, response, db = MEMDB(Client()))
    # ---------------------------------------------------------------------

# -------------------------------------------------------------------------
# by default give a view/generic.extension to all actions from localhost
# none otherwise. a pattern can be 'controller/function.extension'
# -------------------------------------------------------------------------
response.generic_patterns = ['*'] if request.is_local else []
# -------------------------------------------------------------------------
# choose a style for forms
# -------------------------------------------------------------------------
response.formstyle = myconf.get('forms.formstyle')  # or 'bootstrap3_stacked' or 'bootstrap2' or other
response.form_label_separator = myconf.get('forms.separator') or ''

# -------------------------------------------------------------------------
# (optional) optimize handling of static files
# -------------------------------------------------------------------------
# response.optimize_css = 'concat,minify,inline'
# response.optimize_js = 'concat,minify,inline'

# -------------------------------------------------------------------------
# (optional) static assets folder versioning
# -------------------------------------------------------------------------
# response.static_version = '0.0.0'

# -------------------------------------------------------------------------
# Here is sample code if you need for
# - email capabilities
# - authentication (registration, login, logout, ... )
# - authorization (role based authorization)
# - services (xml, csv, json, xmlrpc, jsonrpc, amf, rss)
# - old style crud actions
# (more options discussed in gluon/tools.py)
# -------------------------------------------------------------------------

from gluon.tools import Auth, Service, PluginManager

# host names must be a list of allowed host names (glob syntax allowed)
auth = Auth(db, host_names=myconf.get('host.names'))
service = Service()
plugins = PluginManager()

# -------------------------------------------------------------------------
# create all tables needed by auth if not custom tables
# -------------------------------------------------------------------------
auth.define_tables(username=False, signature=False)

# -------------------------------------------------------------------------
# configure email
# -------------------------------------------------------------------------
mail = auth.settings.mailer
mail.settings.server = 'logging' if request.is_local else myconf.get('smtp.server')
mail.settings.sender = myconf.get('smtp.sender')
mail.settings.login = myconf.get('smtp.login')
mail.settings.tls = myconf.get('smtp.tls') or False
mail.settings.ssl = myconf.get('smtp.ssl') or False

# -------------------------------------------------------------------------
# configure auth policy
# -------------------------------------------------------------------------
auth.settings.registration_requires_verification = False
auth.settings.registration_requires_approval = False
auth.settings.reset_password_requires_verification = True

# -------------------------------------------------------------------------
# Define your tables below (or better in another model file) for example
#
# >>> db.define_table('mytable', Field('myfield', 'string'))
#
# Fields can be 'string','text','password','integer','double','boolean'
#       'date','time','datetime','blob','upload', 'reference TABLENAME'
# There is an implicit 'id integer autoincrement' field
# Consult manual for more options, validators, etc.
#
# More API examples for controllers:
#
# >>> db.mytable.insert(myfield='value')
# >>> rows = db(db.mytable.myfield == 'value').select(db.mytable.ALL)
# >>> for row in rows: print row.id, row.myfield
# -------------------------------------------------------------------------

# -------------------------------------------------------------------------
# after defining tables, uncomment below to enable auditing
# -------------------------------------------------------------------------
# auth.enable_record_versioning(db)

# ----------------------------------------------------------------------------------------------------------------------
# Usage of new settings
# ----------------------------------------------------------------------------------------------------------------------
auth.settings.login_button_icon = 'power'
auth.settings.register_button_icon = 'add user'
auth.settings.lost_password_button_icon = 'lock'
auth.settings.forgot_username_button_icon = 'edit'
auth.settings.change_password_button_icon = 'lock'
auth.settings.profile_buttons_icon = 'user'
auth.settings.logout_button_icon = 'power'
auth.settings.default_scafolding = 'semantic2'


# ----------------------------------------------------------------------------------------------------------------------
# Custom dirty MENU helper
# ----------------------------------------------------------------------------------------------------------------------
class MENU(DIV):
    """
    Used to build menus

    Args:
        _class: defaults to 'web2py-menu web2py-menu-vertical'
        ul_class: defaults to 'web2py-menu-vertical'
        li_class: defaults to 'web2py-menu-expand'
        li_first: defaults to 'web2py-menu-first'
        li_last: defaults to 'web2py-menu-last'

    Use like::

        menu = MENU([['name', False, URL(...), [submenu]], ...])
        {{=menu}}

    """

    tag = 'ul'

    def __init__(self, data, **args):
        self.data = data
        self.attributes = args
        self.components = []
        if '_class' not in self.attributes:
            self['_class'] = 'web2py-menu web2py-menu-vertical'
        if 'ul_class' not in self.attributes:
            self['ul_class'] = 'web2py-menu-vertical'
        if 'li_class' not in self.attributes:
            self['li_class'] = 'web2py-menu-expand'
        if 'li_first' not in self.attributes:
            self['li_first'] = 'web2py-menu-first'
        if 'li_last' not in self.attributes:
            self['li_last'] = 'web2py-menu-last'
        if 'li_active' not in self.attributes:
            self['li_active'] = 'web2py-menu-active'
        if 'mobile' not in self.attributes:
            self['mobile'] = False

    def serialize(self, data, level=0):
        if level == 0:
            menu_dropdowns = []
        else:
            menu_dropdowns = []
        for item in data:
            if isinstance(item, DIV):
                menu_dropdowns.append(item)
            else:
                (name, active, link) = item[:3]
                if isinstance(link, DIV):
                    menu_entry = link
                    print name, 'DIV'
                elif 'no_link_url' in self.attributes and self['no_link_url'] == link:
                    menu_entry = DIV(name, _class='item')
                    print name, 'no_link_url'
                elif isinstance(link, dict):
                    menu_entry = A(name, _class='item', **link)
                    print name, 'dict'
                elif link:
                    menu_entry = A(name, _href=link, _class='item')
                    print name, 'elif'
                elif not link and isinstance(name, A):
                    menu_entry = name
                    print name, 'elif not link'
                else:
                    menu_entry = A(name, _href='#', _onclick='javascript:void(0);return false;', _class='item')
                    print name, 'else'
                if level == 0 and item == data[0]:
                    menu_entry['_class'] = self['li_first'] + ' item'
                    print name, 'level 0'
                elif level == 0 and item == data[-1]:
                    menu_entry['_class'] = self['li_last'] + ' item'
                    print name, 'level 0 data -1'
                if len(item) > 3 and item[3]:
                    menu_entry = DIV(name, _class='ui simple dropdown item')
                    menu_entry.append(DIV(self.serialize(item[3], level + 1), _class='menu'))
                    print name, 'recursion'
                if active or ('active_url' in self.attributes and self['active_url'] == link):
                    if menu_entry['_class']:
                        menu_entry['_class'] = menu_entry['_class'] + ' ' + self['li_active']
                    else:
                        menu_entry['_class'] = self['li_active']
                if len(item) <= 4 or item[4] == True:
                    menu_dropdowns.append(menu_entry)
        return CAT(menu_dropdowns)

    def serialize_mobile(self, data, select=None, prefix=''):
        if not select:
            select = SELECT(**self.attributes)
        custom_items = []
        for item in data:
            # Custom item aren't serialized as mobile
            if len(item) >= 3 and (not item[0]) or (isinstance(item[0], DIV) and not (item[2])):
                # ex: ('', False, A('title', _href=URL(...), _title="title"))
                # ex: (A('title', _href=URL(...), _title="title"), False, None)
                custom_items.append(item)
            elif len(item) <= 4 or item[4] == True:
                select.append(OPTION(CAT(prefix, item[0]),
                                     _value=item[2], _selected=item[1]))
                if len(item) > 3 and len(item[3]):
                    self.serialize_mobile(
                        item[3], select, prefix=CAT(prefix, item[0], '/'))
        select['_onchange'] = 'window.location=this.value'
        # avoid to wrap the select if no custom items are present
        html = DIV(select, self.serialize(custom_items)) if len(custom_items) else select
        return html

    def xml(self):
        if self['mobile']:
            return self.serialize_mobile(self.data, 0).xml()
        else:
            return self.serialize(self.data, 0).xml()

